//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

package com.ibm.as400.opnav.MsgQueueSample1;

// Import the opnav package
import com.ibm.as400.opnav.*;

// Import the PDML runtime
import com.ibm.as400.ui.framework.java.*;

// Miscellaneous Sun classes
import javax.swing.JOptionPane;
import java.awt.Frame;
import java.text.MessageFormat;

/**
 * The <code>DataBean</code> which manages data for the "Confirm Delete"
 * panel.  See <strong>com.ibm.as400.opnav.DataBean.</strong>
 */
public class MqDeleteMessageBean extends Object
    implements DataBean
{
    /**
     * The list of messages selected by the user to be deleted.
     * Generated by the GUI Builder.
     */
    private String[] m_sMessage;

    /**
     * The list of messages to be deleted.
     * Generated by the GUI Builder.
     */
    private ItemDescriptor[] m_idMessage;

    /**
     * The list of MqMessage objects for the messages.
     */
    MqMessage[]   m_deleteList;

    /**
     * The owning frame for any messages that are displayed.
     */
    Frame           m_owner;

    /**
     * Lets clients determine whether messages were actually deleted.
     */
    boolean     m_actionPerformed = false;

    /**
     * Lets clients determine how many messages were deleted.
     */
    int         m_deleteCount = 0;

    // Use the ResourceLoader for loading MRI from the sample resource bundle
    static private ResourceLoader  m_loader = new ResourceLoader();

    static
    {
        // Set up to load MRI.  We do this in the static initializer for
        // the class so that we only load the resource bundle once.
        m_loader.setResourceName("com.ibm.as400.opnav.MsgQueueSample1.MessageQueueGUI");
    }

    /**
     * Constructs an <code>MqDeleteMessageBean</code>.
     *
     * @param objList The list of messages to be deleted.
     * @param owner The parent frame for any messages that are displayed.
     */
    MqDeleteMessageBean(MqMessage[] objList, Frame owner)
    {
        // Store the list of objects to be deleted
        m_deleteList = objList;

        // Store the owning frame
        m_owner = owner;
    }

    /**
     * Returns the <strong>Message</strong> list attribute.
     * Generated by the GUI Builder.
     *
     * @return The list of messages to be deleted.
     */
    public ItemDescriptor[] getMessageList()
    {
        return m_idMessage;
    }

    /**
     * Sets the <strong>Message</strong> list attribute.
     * Generated by the GUI Builder.
     *
     * @param items The list of messages to be deleted.
     */
    public void setMessageList(ItemDescriptor[] items)
    {
        m_idMessage = items;
    }

    /**
     * Returns the <strong>Message</strong> selection attribute.
     * Generated by the GUI Builder.
     *
     * @return The messages in the list which should be highlighted.
     */
    public String[] getMessageSelection()
    {
        return m_sMessage;
    }

    /**
     * Sets the <strong>Message</strong> selection attribute.
     * Generated by the GUI Builder.
     *
     * @param select The messages in the list which should be highlighted.
     */
    public void setMessageSelection(String[] select)
    {
        m_sMessage = select;
    }

    /**
     * Returns a <code>Capabilities</code> object which identifies
     * the attributes that aren't supported by this <code>DataBean</code>.
     * See <strong>com.ibm.as400.ui.framework.java.Capabilities.</strong>
     *
     * There are many reasons why a <code>DataBean</code> might not support 
     * a given attribute.  For example:
     * <p>
     * <ul>
     * <li>The attribute may not be supported by the server platform.
     * <li>The attribute may not be supported by the operating system version 
     * that is running on the server.
     * <li>The attribute may be supported, but the current user of
     * the client application may not be authorized to change the attribute.
     * </ul>
     * <p>
     * The <code>Capabilities</code> object may provide a list
     * of instructions, or <code>HandlerTask</code>s, which the
     * UI framework is to perform.  Typically these involve disabling
     * or removing certain UI components from a panel.
     *
     * @return  a <code>Capabilities</code> object which reflects this object's capabilities
     */
    public Capabilities getCapabilities()
    {
        // No restrictions are needed
        return null;
    }

    /**
     * Validates any changes that have been applied to this object via
     * its accessor methods.
     * See <strong>com.ibm.as400.opnav.DataBean.</strong>
     * <p>
     * Called just prior to <code>save</code> to give
     * the receiver an opportunity to verify that its data is consistent
     * with any changes made to other <code>DataBean</code>s associated
     * with a given panel.  If an inconsistency is discovered the
     * <code>DataBean</code> should throw an exception.  This will
     * prevent the changes from being committed.
     */
    public void verifyChanges()
    {
        // No processing required
    }

    /**
     * Saves all changes that have been applied to this object via
     * its accessor methods.  Called when the user has clicked the
     * <kbd>OK</kbd> button on a panel.
     * See <strong>com.ibm.as400.opnav.DataBean.</strong>
     */
    public void save()
    {
        // If no messages are selected just quit
        if (m_sMessage.length == 0)
            return;

        // Indicate messages were deleted
        m_actionPerformed = true;
  
        // Iterate over the delete list, deleting the messages that are selected
        for (int i = 0; i < m_deleteList.length; i++)
        {
            // Get the next message object
            MqMessage message = m_deleteList[i];

            // See if this is message is selected
            for (int j = 0; j < m_sMessage.length; j++)
            {
                if (m_sMessage[j].equals(message.getText()))
                {
                    try { message.getQueue().remove(message.getKey()); }
                    catch (Exception e)
                    {
                        // Log the error
                        Monitor.logThrowable(e);

                        // Load the error message text
                        String msg = m_loader.getString("error.text.delete.msg");

                        // Substitution text is the system name
                        Object[] args = { message.getQueue().getSystem().getSystemName() };

                        // Format the error message
                        String usrMsg = MessageFormat.format(msg, args);

                        // Display it to the user
                        MessageBoxDialog.showMessageDialog(m_owner,
                                                           usrMsg,
                                                           m_loader.getString("error.title.msgbox"),
                                                           JOptionPane.WARNING_MESSAGE);

                        // Remaining deletes probably won't work either
                        return;
                    }

                    // Increment the delete count
                    m_deleteCount++;

                    // Process the next message
                    break;
                }
            }
        }
    }

    /**
     * Initializes this object.
     * See <strong>com.ibm.as400.opnav.DataBean.</strong>
     * <p>
     * This method should be called prior to handing the <code>DataBean</code>
     * off to the UI Framework.  On return from this method the bean
     * should be in a state where it is ready to return data on any of
     * its accessor methods.
     */
    public void load()
    {
        // Build the item descriptor list for transmission to the UI
        m_idMessage = new ItemDescriptor[m_deleteList.length];

        for (int i = 0; i < m_deleteList.length; i++)
        {
            MqMessage msg = m_deleteList[i];
            m_idMessage[i] = new ItemDescriptor(msg.getText(), msg.getText());
        }

        // Select all the items
        m_sMessage = new String[m_deleteList.length];

        for (int i = 0; i < m_deleteList.length; i++)
        {
            MqMessage msg = m_deleteList[i];
            m_sMessage[i] = msg.getText();
        }
    }
}
