//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

package com.ibm.as400.opnav.MsgQueueSample3;

// Import the opnav package
import com.ibm.as400.opnav.*;

// Import the PDML runtime
import com.ibm.as400.ui.framework.java.*;

// Import the access classes we need
import com.ibm.as400.access.AS400;
import com.ibm.as400.access.MessageQueue;

// Miscellaneous Sun classes
import javax.swing.JOptionPane;
import java.awt.Frame;
import java.text.MessageFormat;

/**
 * The <code>DataBean</code> which manages data for the "New Message"
 * panel.  See <strong>com.ibm.as400.opnav.DataBean.</strong>
 */
public class MqNewMessageBean extends Object
    implements DataBean
{
    /**
     * The message to be added. Generated by the GUI Builder.
     */
    private String  m_sMessage;

    /**
     * The Toolbox object which will be used to add the new message.
     */
    MessageQueue    m_queue;

    /**
     * The owning frame for any messages that are displayed.
     */
    Frame           m_owner;

    /**
     * Lets clients determine whether a new message was actually added.
     */
    boolean         m_actionPerformed = false;

    // Use the ResourceLoader for loading MRI from the sample resource bundle
    static private ResourceLoader  m_loader = new ResourceLoader();

    static
    {
        // Set up to load MRI.  We do this in the static initializer for
        // the class so that we only load the resource bundle once.
        m_loader.setResourceName("com.ibm.as400.opnav.MsgQueueSample3.MessageQueueGUI");
    }

    /**
     * Constructs an <code>MqNewMessageBean</code>.  Parameters on the constructor
     * identify the system and the message queue on which the new message will
     * be created.  If the message queue name is null, the message will be
     * created on the user's default message queue.
     *
     * @param as400 The system object for the system on which the message queue resides.
     * @param path  The fully-qualified IFS pathname of the message queue (may be null).
     * @param owner The parent frame for any dialogs that are displayed.
     */
    public MqNewMessageBean(AS400 as400, String path, Frame owner)
    {
        // If no queue name is specified, use the user's default
        String queueName = (path != null ? path : MessageQueue.CURRENT);

        // Create the Toolbox message queue object
        m_queue = new MessageQueue(as400, queueName);

        // Store the owning frame
        m_owner = owner;
    }

    /**
     * Returns the <strong>Message</strong> attribute.
     * Generated by the GUI Builder.
     *
     * @return The text of the message to be created.
     */
    public String getMessage()
    {
        return m_sMessage;
    }

    /**
     * Sets the <strong>Message</strong> attribute.
     * Generated by the GUI Builder.
     *
     * @param s The text of the message to be created.
     */
    public void setMessage(String s)
    {
        m_sMessage = s;
    }

    /**
     * Returns a <code>Capabilities</code> object which identifies
     * the attributes that aren't supported by this <code>DataBean</code>.
     * See <strong>com.ibm.as400.ui.framework.java.Capabilities.</strong>
     *
     * There are many reasons why a <code>DataBean</code> might not support 
     * a given attribute.  For example:
     * <p>
     * <ul>
     * <li>The attribute may not be supported by the server platform.
     * <li>The attribute may not be supported by the operating system version 
     * that is running on the server.
     * <li>The attribute may be supported, but the current user of
     * the client application may not be authorized to change the attribute.
     * </ul>
     * <p>
     * The <code>Capabilities</code> object may provide a list
     * of instructions, or <code>HandlerTask</code>s, which the
     * UI framework is to perform.  Typically these involve disabling
     * or removing certain UI components from a panel.
     *
     * @return  a <code>Capabilities</code> object which reflects this object's capabilities
     */
    public Capabilities getCapabilities()
    {
        // No restrictions are needed
        return null;
    }

    /**
     * Validates any changes that have been applied to this object via
     * its accessor methods.
     * See <strong>com.ibm.as400.opnav.DataBean.</strong>
     * <p>
     * Called just prior to <code>save</code> to give
     * the receiver an opportunity to verify that its data is consistent
     * with any changes made to other <code>DataBean</code>s associated
     * with a given panel.  If an inconsistency is discovered the
     * <code>DataBean</code> should throw an exception.  This will
     * prevent the changes from being committed.
     */
    public void verifyChanges()
    {
        // No processing required
    }

    /**
     * Saves all changes that have been applied to this object via
     * its accessor methods.  Called when the user has clicked the
     * <kbd>OK</kbd> button on a panel.
     * See <strong>com.ibm.as400.opnav.DataBean.</strong>
     */
    public void save()
    {
        // Send the new message to the message queue
        try { m_queue.sendInformational(m_sMessage); }
        catch (Exception e)
        {
            // Log the error
            Monitor.logThrowable(e);

            // Load the error message text
            String msg = m_loader.getString("error.text.send.msg");

            // Substitution text is the system name
            Object[] args = { m_queue.getSystem().getSystemName() };

            // Format the error message
            String usrMsg = MessageFormat.format(msg, args);

            // Display it to the user
            MessageBoxDialog.showMessageDialog(m_owner,
                                               usrMsg,
                                               m_loader.getString("error.title.msgbox"),
                                               JOptionPane.WARNING_MESSAGE);

            return;
        }

        // Indicate create performed
        m_actionPerformed = true;
    }

    /**
     * Initializes this object.
     * See <strong>com.ibm.as400.opnav.DataBean.</strong>
     * <p>
     * This method should be called prior to handing the <code>DataBean</code>
     * off to the UI Framework.  On return from this method the bean
     * should be in a state where it is ready to return data on any of
     * its accessor methods.
     */
    public void load()
    {
        m_sMessage = "";
    }
}
