//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

package com.ibm.as400.opnav.MsgQueueSample4;

//Import the entire opnav package
import com.ibm.as400.opnav.*;

import com.ibm.as400.access.AS400;
import com.ibm.as400.access.QSYSObjectPathName;

import com.ibm.as400.ui.framework.java.MessageBoxDialog;

import javax.swing.JOptionPane;

import java.awt.Component;
import java.awt.Frame;
import java.awt.Point;

import java.text.MessageFormat;

/**
 * The <code>DropTargetManager</code> implementation for this plug-in.
 * <p>
 * A <code>DropTargetManager</code> implementation provides an indication
 * of the type of actions supported by the drop target, based on the type
 * of object(s) being dropped and the keyboard/mouse button state.  It then
 * performs the appropriate action on the source object(s) upon completion
 * of the drop operation.
 * See <strong>com.ibm.as400.opnav.DropTargetManager.</strong>
 * <p>
 * <code>MqDropTargetManager</code> handles dragging and dropping of one
 * or more messages from one message queue to another. The messages are
 * added to the target queue and deleted from the source.
 */
public class MqDropTargetManager implements DropTargetManager
{
    /**
     * The target of the drop operation.
     */
	ObjectName		m_dropTarget;

    /**
     * The object type of the drop target.
     */
    String          m_targetType;

    /**
     * The list of objects being dragged.
     */
	ObjectName[]    m_sourceObjects;

    /**
     * The owning frame for any dialog windows displayed
     * while processing a drop.
     */
    Frame           m_owner;

    // Create a shared UIServices object to improve performance
	private UIServices		m_UI = new UIServices();

    // Use the ResourceLoader for loading MRI from the sample resource bundle
    static private ResourceLoader  m_loader = new ResourceLoader();

    static
    {
        // Set up to load MRI.  We do this in the static initializer for
        // the class so that we only load the resource bundle once.
        m_loader.setResourceName("com.ibm.as400.opnav.MsgQueueSample4.MessageQueueGUI");

        // Set the service log file name in production
//      try { Monitor.setLogFileName(Monitor.SHARED_LOG_FILE_NAME); }
        try { Monitor.setLogFileName(null); }
        catch (java.io.IOException e) {}
    }

    /**
     * Identifies the target of the drag/drop operation.
     *
     * This method is called immediately after this object is instantiated.
     * See <strong>com.ibm.as400.opnav.DropTargetManager.</strong>
     *
  	 * @param dropTarget    The object name of the drop target.
     * @param owner         The frame to be specified as the owner of any dialog panels that are displayed.
     */
	public void initialize(ObjectName dropTarget, Frame owner)
	{
        // Save the name of the drop target
		m_dropTarget = dropTarget;

		// Get the type of the target
		try { m_targetType = m_dropTarget.getObjectType(); }
		catch (ObjectNameException e)
		{
            // Error unlikely in production
		    Monitor.logThrowable(e);
		}

        // Save the owner frame
        m_owner = owner;
	}

    /**
     * Indicates whether a drop can be accepted, and if so, the action that
  	 * will take place on the drop.
     * <p>
     * This method is called the first time the user drags the mouse over
     * the UI representation of the drop target.  <code>dragEnter</code>
  	 * must determine what actions will be accepted, if any, based on
     * the actions supported by the drag source and the state of the
     * modifier keys.
     * See <strong>com.ibm.as400.opnav.DropTargetManager.</strong>
     *
  	 * @param sourceObjects  An array of <code>ObjectNames</code> which identify
     * the objects being dragged.
     * @param target	The UI component representing the drop target. (unimplemented)
     * @param location	The location of the drag cursor over the drop target. (unimplemented)
     * @param modifiers	Flags indicating the keyboard/mouse state.
     * @param sourceActions	Flags indicating which actions the drop source supports.
     * @return    The action flag corresponding to the action desired by the drop target.
     */
	public int dragEnter(ObjectName[] sourceObjects, Component target, Point location, int modifiers, int sourceActions)
	{
		// Save the source objects
		m_sourceObjects = sourceObjects;

		// Default is to return no actions supported
		int result = DropTargetManager.ACTION_NONE;

		// Get the type of objects we are dealing with
		String sourceType;

        // NOTE: We assume all objects are of the same type
		try { sourceType = sourceObjects[0].getObjectType(); }
		catch (ObjectNameException e)
		{
		    Monitor.logThrowable(e);
		    return result;
		}

        // Enable dropping of a message object onto a message queue
        if (sourceType.equals("Message4") && m_targetType.equals("MessageQueue4"))
		{
			result = DropTargetManager.ACTION_MOVE;
			try { m_UI.updateStatusArea(m_owner, "Moves the message to the target message queue."); }
			catch (UIServicesException e) {}
		}

		return result;
	}

    /**
     * Called while the user continues to drag the mouse over
     * the UI representation of the drop target.  <code>dragOver</code>
     * must determine what actions will be accepted, if any, based on
     * the actions supported by the drag source and the state of the
     * modifier keys.
     *
     * @param target	The UI component representing the drop target. (unimplemented)
     * @param location	The location of the drag cursor over the drop target. (unimplemented)
     * @param modifiers	Flags indicating the keyboard/mouse state.
     * @param sourceActions	Flags indicating which actions the drop source supports.
     * @return    The action flag corresponding to the action desired by the drop target.
     */
	public int dragOver(Component target, Point location, int modifiers, int sourceActions)
	{
		// Default is to return no actions supported
		int result = DropTargetManager.ACTION_NONE;

		// Get the type of objects we are dealing with
		String sourceType;

        // NOTE: We assume all objects are of the same type
		try { sourceType = m_sourceObjects[0].getObjectType(); }
		catch (ObjectNameException e)
		{
		    Monitor.logThrowable(e);
		    return result;
		}

        // Continue to enable dropping of a message object onto a message queue
        if (sourceType.equals("Message4") && m_targetType.equals("MessageQueue4"))
		{
			result = DropTargetManager.ACTION_MOVE;
			try { m_UI.updateStatusArea(m_owner, "Moves the message to the target message queue."); }
			catch (UIServicesException e) {}
		}

		return result;
	}

    /**
     * Called when the user ceases to drag the mouse over
     * the UI representation of the drop target.
     */
	public void dragExit()
	{
		// Clean up the source/target objects
		m_dropTarget = null;
		m_sourceObjects = null;

        // Clear the status area
		try { m_UI.updateStatusArea(m_owner, ""); }
		catch (UIServicesException e) {}
	}

    /**
     * Called when the user "drops" the objects being dragged
     * by releasing the mouse button.  <code>drop</code> may either reject
     * the drop operation or perform the appropriate action(s) on the drop target,
     * based on the state of the modifier keys.
     *
     * @param sourceObjects  An array of <code>ObjectNames</code> which identify
     * the objects being dropped.
     * @param target	The UI component representing the drop target. (unimplemented)
     * @param location	The location of the drag cursor over the drop target. (unimplemented)
     * @param modifiers	Flags indicating the keyboard/mouse state.
     * @param sourceActions	Flags indicating which actions the drop source supports.
     * @return    The action flag corresponding to the action performed by the drop target.
     */
	public int drop(ObjectName[] sourceObjects, Component target, Point location, int modifiers, int sourceActions)
	{
		// Default is to return no actions performed
		int result = DropTargetManager.ACTION_NONE;

		// Get the type of objects we are dealing with
		String sourceType;

        // NOTE: We assume all objects are of the same type
		try { sourceType = m_sourceObjects[0].getObjectType(); }
		catch (ObjectNameException e)
		{
		    Monitor.logThrowable(e);
		    return result;
		}

        // Verify the source and target are the correct types
        if (sourceType.equals("Message4") && m_targetType.equals("MessageQueue4"))
		{
            boolean actionPerformed = false;

            // To move the messages we first add them to the target
            // queue and then remove them from the source queue
            if (addMessages())
                actionPerformed = deleteMessages();

            // If everything worked, refresh the list view
			if (actionPerformed)
            {
                // Load the status area text
                String message = m_loader.getString("message.text.movemsg");

                // Substitution text is the count of deleted messages and the total count
                Object[] countString = { new Integer(m_sourceObjects.length) };

                // Format the message
                message = MessageFormat.format(message, countString);

                // Refresh the list to show that the message has been moved
                try { new UIServices().refreshList(m_owner, message); }
                catch (UIServicesException e)
                {
                    // Error unlikely
                    Monitor.logThrowable(e);
                }
            }

            result = DropTargetManager.ACTION_MOVE;
        }

		// Clean up the source/target objects
		m_dropTarget = null;
		m_sourceObjects = null;

		return result;
	}

    private boolean addMessages()
    {
        // If an error occurs we need the name of the AS/400 we're trying to connect to
        String systemName = "UNKNOWN";

        // Get the name of the AS/400 associated with this drop target
        try { systemName = m_dropTarget.getSystemName(); }
        catch (ObjectNameException e) {}

        // Get the Toolbox object for the message queue's parent AS/400 system
        AS400 as400 = null;

        try { as400 = (AS400)m_dropTarget.getSystemObject(); }
        catch (ObjectNameException e)
        {
            // Log the error
            Monitor.logThrowable(e);

            // Load the error message text
            String msg = m_loader.getString("error.text.connection.info");

            // Substitution text is the system name
            Object[] args = { systemName };

            // Format the error message
            String usrMsg = MessageFormat.format(msg, args);

            // Display it to the user
            MessageBoxDialog.showMessageDialog(m_owner,
                                               usrMsg,
                                               m_loader.getString("error.title.msgbox"),
                                               JOptionPane.WARNING_MESSAGE);

            return false;
        }

        // Get the name of the target message queue
        String queueName = null;
        try { queueName = m_dropTarget.getDisplayName(); }
        catch (ObjectNameException e)
        {
            // Error unlikely in production
            Monitor.logThrowable(e);
            return false;
        }

        // We create an MqNewMessageBean and use it to send the messages to the target queue
        QSYSObjectPathName path = new QSYSObjectPathName("QUSRSYS", queueName, "MSGQ");
        MqNewMessageBean nmb = new MqNewMessageBean(as400, path.getPath(), m_owner);
        nmb.load();

        // Process all of the source messages
        for (int i = 0; i < m_sourceObjects.length; i++)
        {
            // Get the next source object
            MqMessage msg = null;

            try { msg = (MqMessage)m_sourceObjects[i].getListObject(); }
            catch (ObjectNameException e)
            {
                // Detailed exception was already logged by MqMessagesListManager
                Monitor.logError(e.getMessage());

                // Load the error message text
                String usrMsg = m_loader.getString("error.text.program");

                // Display it to the user
                MessageBoxDialog.showMessageDialog(m_owner,
                                                   usrMsg,
                                                   m_loader.getString("error.title.msgbox"),
                                                   JOptionPane.WARNING_MESSAGE);

                return false;
            }

            // Extract the message text
            nmb.setMessage(msg.getText());

            // Add it to the target message queue
            nmb.save();
            if (!nmb.m_actionPerformed)
                return false;
        }

        return true;
    }

    private boolean deleteMessages()
    {
        // Build a list of the selected objects
        MqMessage[] objList = new MqMessage[m_sourceObjects.length];

        // Get the list objects for the items that are selected
        for (int i = 0; i < m_sourceObjects.length; i++)
        {
            MqMessage msg = null;

            try { msg = (MqMessage)m_sourceObjects[i].getListObject(); }
            catch (ObjectNameException e)
            {
                // Detailed exception was already logged by MessagesListManager
                Monitor.logError(e.getMessage());

                // Load the error message text
                String usrMsg = m_loader.getString("error.text.program");

                // Display it to the user
                MessageBoxDialog.showMessageDialog(m_owner,
                                                   usrMsg,
                                                   m_loader.getString("error.title.msgbox"),
                                                   JOptionPane.WARNING_MESSAGE);

                return false;
            }

            objList[i] = msg;
        }

        // We create an MqDeleteMessageBean and use it to delete the messages from the source queue
        MqDeleteMessageBean dmb = new MqDeleteMessageBean(objList, m_owner);
        dmb.load();

        // Delete the messages
        dmb.save();

        return true;
    }
}
