//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

package com.ibm.as400.opnav.MsgQueueSample5;

import java.awt.datatransfer.*;
import java.io.*;

/**
 * This class is copied verbatim from Core Java 1.1 Volume 2,
 * Chapter 6. It handles the BASE64 encoding of Java objects
 * as strings, and the transfer of strings to/from the
 * system clipboard.
 */
class MimeClipboard extends Clipboard
{
    public MimeClipboard(Clipboard cb)
    {
        super("MIME/" + cb.getName());
        clip = cb;
    }
    
    public synchronized void setContents(Transferable contents, ClipboardOwner owner)
    {   
        // If the contents specified are the recognized Transferable
        // implementation, encode the data and place it on the clipboard.
        // Otherwise, just place the data on the clipboard.
        if(contents instanceof SerializableSelection)
        {
            try
            {
                // Get the object to be transferred
                DataFlavor flavor = SerializableSelection.serializableFlavor;
                Serializable obj = (Serializable)contents.getTransferData(flavor);

                // Encode it as a string
                String enc = encode(obj);

                // Construct a custom header
                String header = "Opnav-plugin:  "
                                + flavor.getMimeType()
                                + "\nContent-length:  "
                                + enc.length() + "\n\n";

                // Create a new StringSelection containing the header and data
                StringSelection selection = new StringSelection(header + enc);
                
                // Place the data on the clipboard
                clip.setContents(selection, owner);
            }
            catch(UnsupportedFlavorException e)
            { e.printStackTrace(); }
            catch(IOException e)
            { e.printStackTrace(); }
        }
        else
            clip.setContents(contents, owner);
    }
    
    public synchronized Transferable getContents(Object requestor)
    {
        // Retrieve the Transferable object on the clipboard
        Transferable contents = clip.getContents(requestor);

        // If the contents retrieved are the desired Transferable
        // implementation, decode the data and place it on the clipboard.
        // Otherwise, just place the data on the clipboard.
        // If the DataFlavor is not supported, or the string on the
        // clipboard doesn't begin as we expect it to, the data on
        // the clipboard is just returned.
        if (contents instanceof StringSelection)
        {
            String data = null;
            try
            {
                data = (String)contents.getTransferData(DataFlavor.stringFlavor);
            }
            catch(UnsupportedFlavorException e)
            { 
                e.printStackTrace();
                return contents; 
            }
            catch(IOException e)
            { 
                e.printStackTrace();
                return contents; 
            }
                        
            if (!data.startsWith("Opnav-plugin:  "))
                return contents;

            // Skip 3 newlines
            int start = -1;
            for(int i = 0; i < 3; i++)
            {
                start = data.indexOf('\n', start + 1);
                if(start < 0) 
                    return contents;
            }

            // The data on the clipboard is usable to us, so decode it
            Serializable obj = decode(data, start);

            // Wrapper the reconstructed object with our Transferable implementation
            SerializableSelection selection = new SerializableSelection(obj);
            return selection;
        }
        else
            return contents;
    }
                            
    private static String encode(Serializable obj)
    {
        try
        {
            StringBuffer sbuf = new StringBuffer();
            Base64OutputStream bout = new Base64OutputStream(sbuf);
            ObjectOutputStream out = new ObjectOutputStream(bout);
            out.writeObject(obj);
            out.flush();
            return sbuf.toString();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            return "";
        }
    }
    
    private static Serializable decode(String s, int start)
    {
        try
        {
            Base64InputStream bin = new Base64InputStream(s, start);
            ObjectInputStream in = new ObjectInputStream(bin);
            Object out = in.readObject();
            return (Serializable)out;
        }
        catch(Exception e)
        {
            e.printStackTrace();
            return null;
        }
    }
    
    private Clipboard clip;
}
    
class Base64OutputStream extends OutputStream
{  
    public Base64OutputStream(StringBuffer sb)
    {  
        sbuf = sb;
    }

    public void write(int c) throws IOException
    {  
        inbuf[i] = c;
        i++;
        if (i == 3)
        {  
            sbuf.append(toBase64[(inbuf[0] & 0xFC) >> 2]);
            
            sbuf.append(toBase64[((inbuf[0] & 0x03) << 4) |
                                 ((inbuf[1] & 0xF0) >> 4)]);
                                     
            sbuf.append(toBase64[((inbuf[1] & 0x0F) << 2) |
                                 ((inbuf[2] & 0xC0) >> 6)]);
                                      
            sbuf.append(toBase64[inbuf[2] & 0x3F]);   
            col += 4;
            i = 0;
            if (col >= 76)
            {  
                sbuf.append('\n');
                col = 0;
            }
        }
    }

    public void flush()
    {  
        if (i == 1)
        {  
            sbuf.append(toBase64[(inbuf[0] & 0xFC) >> 2]);
            sbuf.append(toBase64[(inbuf[0] & 0x03) << 4]);
            sbuf.append('=');
            sbuf.append('=');
        }
        else if (i == 2)
        {  
            sbuf.append(toBase64[(inbuf[0] & 0xFC) >> 2]);
        
            sbuf.append(toBase64[((inbuf[0] & 0x03) << 4) |
                                 ((inbuf[1] & 0xF0) >> 4)]);
                             
            sbuf.append(toBase64[(inbuf[1] & 0x0F) << 2]);
            sbuf.append('=');
        }
        sbuf.append('\n');
    }

    private static char[] toBase64 =
    {  
        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
        'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
        'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
        'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
        'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
        'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
        'w', 'x', 'y', 'z', '0', '1', '2', '3',
        '4', '5', '6', '7', '8', '9', '+', '/'
    };
   
    StringBuffer sbuf;
    int col = 0;
    int i = 0;
    int[] inbuf = new int[3];
}

class Base64InputStream extends InputStream
{  
    public Base64InputStream(String s, int start)
    {  
        str = s;
        pos = start;
        i = 0;
    }

    public int read() 
    {  
        while (pos < str.length() && 
               Character.isWhitespace(str.charAt(pos)))
        {  pos++; }
        if (pos >= str.length()) 
            return -1;
        if (i == 0)
        {  
            int ch1 = str.charAt(pos) & 0x7F;
            int ch2 = str.charAt(pos + 1) & 0x7F;
            i++;
            return (fromBase64[ch1] << 2) | (fromBase64[ch2] >> 4);
        }
        else if (i == 1)
        {  
            int ch1 = str.charAt(pos + 1) & 0x7F;
            int ch2 = str.charAt(pos + 2) & 0x7F;
            if (ch2 == '=') return -1;
            i++;
            return ((fromBase64[ch1] & 0x0F) << 4) | (fromBase64[ch2] >> 2);
        }
        else
        {  
            int ch1 = str.charAt(pos + 2) & 0x7F;
            int ch2 = str.charAt(pos + 3) & 0x7F;
            if (ch2 == '=') return -1;
            i = 0;
            pos += 4;
            return ((fromBase64[ch1] & 0x03) << 6) | fromBase64[ch2];
        }
   }

    private static int[] fromBase64 =
    { 
        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, 62, -1, -1, -1, 63,
        52, 53, 54, 55, 56, 57, 58, 59,
        60, 61, -1, -1, -1, -1, -1, -1,
        -1,  0,  1,  2,  3,  4,  5,  6,
         7,  8,  9, 10, 11, 12, 13, 14,
        15, 16, 17, 18, 19, 20, 21, 22,
        23, 24, 25, -1, -1, -1, -1, -1,
        -1, 26, 27, 28, 29, 30, 31, 32,
        33, 34, 35, 36, 37, 38, 39, 40,
        41, 42, 43, 44, 45, 46, 47, 48,
        49, 50, 51, -1, -1, -1, -1, -1
    };

    String str;
    int pos;
    int i;
}
