//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************
package com.ibm.as400.opnav.MsgQueueSample6;

// Import the base GUI framework
import com.ibm.as400.ui.framework.java.*;

// Import the opnav package
import com.ibm.as400.opnav.*;

// Get date formatting
import java.text.DateFormat;

// Import time functions
import java.util.*;

/**
 * The <code>DataBean</code> which manages data for the "Detail" and "Sender"
 * property pages.  See <strong>com.ibm.as400.opnav.DataBean.</strong>
 * <p>
 * MqMessagePropertiesBean is written as one bean per message.  
 * This means one instance of the bean is passed to both pages.
 * Having one bean makes it easier to share data between pages
 * if needed.
 */

public class MqMessagePropertiesBean extends Object
    implements DataBean
{
    // Use the ResourceLoader for loading MRI from the sample resource bundle
    static private ResourceLoader  m_loader = new ResourceLoader();

    static
    {
        // Set up to load MRI.  We do this in the static initializer for
        // the class so that we only load the resource bundle once.
        m_loader.setResourceName("com.ibm.as400.opnav.MsgQueueSample6.MessageQueueGUI");
    }
    
    // The MqMessage this bean is representing
    private MqMessage m_message;
    
    // Private strings for the values extracted from m_message
    private String m_sTimeSent;
    private String m_sDateSent;
    private String m_sSendingUserProfile;
    private String m_sSendingProgramName;
    private String m_sSendingJobNumber;
    private String m_sSendingJob;
    private String m_sMessageIdentifier;
    private String m_sMessageSeverity;
    private String m_sMessageType;
    private String m_sMessageText;
    
    /**
     * Constructor for the bean.  Requires the message object to have properties displayed.
     * Added by hand to the bean generated by the GUI Builder.
     *
     * @message The message to have properties displayed.
     */
    public MqMessagePropertiesBean(MqMessage message)
    {
        m_message = message;
    }

    /**
     * Returns the time the message was sent.
     * Generated by the GUI Builder.
     *
     * @return The time the message was sent.
     */
    public String getTimeSent()
    {
        return m_sTimeSent;
    }

    /**
     * Returns the date the message was sent.
     * Generated by the GUI Builder.
     *
     * @return The date the message was sent.
     */
    public String getDateSent()
    {
        return m_sDateSent;
    }

    /**
     * Returns the user profile that sent the message.
     * Generated by the GUI Builder.
     *
     * @return The user profile that sent the message.
     */
    public String getSendingUserProfile()
    {
        return m_sSendingUserProfile;
    }

    /**
     * Returns the program name that sent the message.
     * Generated by the GUI Builder.
     *
     * @return The program name that sent the message.
     */
    public String getSendingProgramName()
    {
        return m_sSendingProgramName;
    }

    /**
     * Returns the number of the job that sent the message.
     * Generated by the GUI Builder.
     *
     * @return The number of the job that sent the message.
     */
    public String getSendingJobNumber()
    {
        return m_sSendingJobNumber;
    }

    /**
     * Returns the name of the job that sent the message.
     * Generated by the GUI Builder.
     *
     * @return The name of the job that sent the message.
     */
    public String getSendingJob()
    {
        return m_sSendingJob;
    }

    /**
     * Returns the identifier of message.
     * Generated by the GUI Builder.
     *
     * @return The identifier of message.
     */
    public String getMessageIdentifier()
    {
        return m_sMessageIdentifier;
    }

    /**
     * Returns the severity of message.
     * Generated by the GUI Builder.
     *
     * @return The severity of message.
     */
    public String getMessageSeverity()
    {
        return m_sMessageSeverity;
    }

    /**
     * Returns the type of message.
     * Generated by the GUI Builder.
     *
     * @return The type of message.
     */
    public String getMessageType()
    {
        return m_sMessageType;
    }

    /**
     * Returns the text of message.
     * Generated by the GUI Builder.
     *
     * @return The text of message.
     */
    public String getMessageText()
    {
        return m_sMessageText;
    }

    /**
     * Sets the text of message.
     * Generated by the GUI Builder.
     *
     * @param s The text of message.
     */
    public void setMessageText(String s)
    {
        m_sMessageText = s;
    }

    /**
     * Returns a <code>Capabilities</code> object which identifies
     * the attributes that aren't supported by this <code>DataBean</code>.
     * See <strong>com.ibm.as400.ui.framework.java.Capabilities.</strong>
     *
     * There are many reasons why a <code>DataBean</code> might not support
     * a given attribute.  For example:
     * <p>
     * <ul>
     * <li>The attribute may not be supported by the server platform.
     * <li>The attribute may not be supported by the operating system version
     * that is running on the server.
     * <li>The attribute may be supported, but the current user of
     * the client application may not be authorized to change the attribute.
     * </ul>
     * <p>
     * The <code>Capabilities</code> object may provide a list
     * of instructions, or <code>HandlerTask</code>s, which the
     * UI framework is to perform.  Typically these involve disabling
     * or removing certain UI components from a panel.
     *
     * @return  a <code>Capabilities</code> object which reflects this object's capabilities
     */
    public Capabilities getCapabilities()
    {
        // No restrictions are needed
        return null;
    }

    /**
     * Validates any changes that have been applied to this object via
     * its accessor methods.
     * See <strong>com.ibm.as400.opnav.DataBean.</strong>
     * <p>
     * Called just prior to <code>save</code> to give
     * the receiver an opportunity to verify that its data is consistent
     * with any changes made to other <code>DataBean</code>s associated
     * with a given panel.  If an inconsistency is discovered the
     * <code>DataBean</code> should throw an exception.  This will
     * prevent the changes from being committed.
     */
    public void verifyChanges()
    {
        // No processing required
    }

    /**
     * Saves all changes that have been applied to this object via
     * its accessor methods.  Called when the user has clicked the
     * <kbd>OK</kbd> button on a panel.
     * See <strong>com.ibm.as400.opnav.DataBean.</strong>
     */
    public void save()
    {
        // Display only property sheets do not need to be saved
    }


    /**
     * Initializes this object.
     * See <strong>com.ibm.as400.opnav.DataBean.</strong>
     * <p>
     * This method should be called prior to handing the <code>DataBean</code>
     * off to the UI Framework.  On return from this method the bean
     * should be in a state where it is ready to return data on any of
     * its accessor methods.
     */
    public void load()
    {
        // Load values from the message object
        m_sSendingUserProfile = m_message.getUser();
        m_sSendingProgramName = m_message.getFromProgram();
        m_sSendingJobNumber   = m_message.getFromJobNumber();
        m_sSendingJob         = m_message.getFromJobName();
        m_sMessageIdentifier  = m_message.getID();
        m_sMessageText        = m_message.getText();
        
        // Load date and time values
        if (m_message.getDate() != null)
        { 
            DateFormat tf = DateFormat.getTimeInstance();
            m_sTimeSent = tf.format(m_message.getDate().getTime());
            DateFormat df = DateFormat.getDateInstance();
            m_sDateSent = df.format(m_message.getDate().getTime());
        }
        else
        {
            m_sTimeSent = m_loader.getString("properties.text.notset");
            m_sDateSent = m_loader.getString("properties.text.notset");
        }
        
        // Load the severity.  If -1, then the severity is not set
        int severity = m_message.getSeverity();
        if (severity < 0)
            m_sMessageSeverity = m_loader.getString("properties.text.notset");
        else
            m_sMessageSeverity = Integer.toString(severity);

        // Load the type.
        switch (m_message.getType())
        {
            case 1:  
                m_sMessageType = m_loader.getString("properties.text.type.completion");
                break;
            case 2:  
                m_sMessageType = m_loader.getString("properties.text.type.diagnostic");
                break;
            case 4:  
                m_sMessageType = m_loader.getString("properties.text.type.informational");
                break;
            case 5:  
                m_sMessageType = m_loader.getString("properties.text.type.inquiry");
                break;
            case 6:  
                m_sMessageType = m_loader.getString("properties.text.type.senders_copy");
                break;
            case 8:  
                m_sMessageType = m_loader.getString("properties.text.type.request");
                break;
            case 10:  
                m_sMessageType = m_loader.getString("properties.text.type.request_with_prompting");
                break;
            case 14:  
                m_sMessageType = m_loader.getString("properties.text.type.notify");
                break;
            case 15:  
                m_sMessageType = m_loader.getString("properties.text.type.escape");
                break;
            case 21:  
                m_sMessageType = m_loader.getString("properties.text.type.reply_not_validity_checked");
                break;
            case 22:  
                m_sMessageType = m_loader.getString("properties.text.type.reply_validity_checked");
                break;
            case 23:  
                m_sMessageType = m_loader.getString("properties.text.type.reply_message_default_used");
                break;
            case 24:  
                m_sMessageType = m_loader.getString("properties.text.type.reply_system_default_used");
                break;
            case 25:  
                m_sMessageType = m_loader.getString("properties.text.type.reply_from_system_reply_list");
                break;
            default:
                // Error unlikely in production
                m_sMessageType = "";
                Monitor.logError("MqMessagePropertiesBean: Invalid message type: " + m_message.getType());
                break;
        }

    }
}
