//********************************************************************
//                       LICENSE AND DISCLAIMER
//
// This material contains IBM copyrighted sample programming source
// code for your consideration.  This sample code has not been
// thoroughly tested under all conditions.  IBM, therefore, cannot
// guarantee or imply reliability, serviceability, or function.
// IBM provides no program services for this material. This material
// is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
// IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SOME
// JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED WARRANTIES, SO
// THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO EVENT WILL IBM
// BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER
// CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING,
// WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS INTERRUPTION, LOSS
// OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION HANDLING SYSTEM OR
// OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
//
//                              COPYRIGHT
//                              ---------
//               (C) Copyright IBM CORP. 1999
//               All rights reserved.
//               US Government Users Restricted Rights -
//               Use, duplication or disclosure restricted
//               by GSA ADP Schedule Contract with IBM Corp.
//               Licensed Material - Property of IBM
//********************************************************************

package com.ibm.as400.opnav.MsgQueueSample6;

// Import the entire opnav package
import com.ibm.as400.opnav.*;
import com.ibm.as400.opnav.Monitor;

import javax.swing.JOptionPane;
import java.awt.Frame;

import com.ibm.as400.ui.framework.java.*;
import com.ibm.as400.ui.framework.java.PanelManager;

import java.awt.event.ActionListener; 
import java.awt.event.ActionEvent; 

/**
 * The <code>PropertiesManager</code> implementation for this Plug-in.
 * <p>
 * The <code>PropertiesManager</code> implementation provides Operations Navigator
 * with a array of panels to be added to the property sheet being constructed.
 * These panels are placed in the array in the order to be added to the sheet.  
 * See <strong>com.ibm.as400.opnav.PropertiesManager.</strong>
 * <p>
 * <code>MqMessagePropertiesManager</code> is invoked whenever Operations Navigator
 * displays properties for MqMessage objects. 
 */
public class MqMessagePropertiesManager implements PropertiesManager
{
    /**
     * The list of selected objects.
     */
    ObjectName[]    m_objectNames;

    /**
     * The owning frame for any dialog windows displayed
     * while processing an action.
     */
    Frame           m_owner;
    
    // Use the ResourceLoader for loading MRI from the sample resource bundle
    static private ResourceLoader  m_loader = new ResourceLoader();

    static
    {
        // Set up to load MRI.  We do this in the static initializer for
        // the class so that we only load the resource bundle once.
        m_loader.setResourceName("com.ibm.as400.opnav.MsgQueueSample6.MessageQueueGUI");
    }
    
    
    /**
     * Identifies the objects in a list for which the properties are being shown.
     *
     * This method is called immediately after this object is instantiated.
     * <p>
     *
     * @param objectNames    an array of object names.
     */
    public void initialize(ObjectName[] objectNames)
    {
        // Save the array of selected object names
        m_objectNames = objectNames;
        m_owner = new Frame();
        // Display it to the user
        Monitor.logError("MqMessagePropertiesManager:  Initializing");
    }

    /**
     * Returns a list of PanelManager objects representing pages to 
     * be added to the property sheet.  Pages are displayed in the order they are returned.
     *
     * @return    an array of PanelManager objects, each of which describes one page.
     */
     public PanelManager[] getPages()
     {
            // Display it to the user
            Monitor.logError("MqMessagePropertiesManager:  getting pages");
            
            // Make sure exceptions can be piped
            System.setErr(System.out);                   	
                          	   
            // Instantiate the panel data bean,used for both panels
            MqMessage msg = null;

            // Get the first MqMessage object from the object list. 
            // Operations Navigator assures there is only one object for property sheets.
            // (If there are more objects, ignore them.)
            try { msg = (MqMessage)m_objectNames[0].getListObject(); }
            catch (ObjectNameException e)
            {
                // Detailed exception was already logged by MqMessagesListManager
                Monitor.logError(e.getMessage());

                // Load the error message text
                String usrMsg = m_loader.getString("error.text.program");

                // Display it to the user
                MessageBoxDialog.showMessageDialog(m_owner,
                                                   usrMsg,
                                                   m_loader.getString("error.title.msgbox"),
                                                   JOptionPane.WARNING_MESSAGE);

                return null;
            }
            
            // Create the bean.  The MqMessagePropertiesBean represents one Message object,
            // but is used for both dialogs, since they together represent the properties
            // of one object.
            MqMessagePropertiesBean dataBean = new MqMessagePropertiesBean(msg);
            dataBean.load();
                            	
            DataBean[] dataBeans = { dataBean };
              
            Frame frame = new Frame();
                          
            // Create the panels
            PanelManager pm_detail = null;
            PanelManager pm_sender = null;
            try
            {
                pm_detail = new PanelManager("com.ibm.as400.opnav.MsgQueueSample6.MessageQueueGUI",
                "IDD_MSG_PROP_DETAIL",
                dataBeans);
                
                pm_sender = new PanelManager("com.ibm.as400.opnav.MsgQueueSample6.MessageQueueGUI",
                "IDD_MSG_PROP_SENDER",
                dataBeans);
                
                // Set the titles of the panels, use the title set in the pdml      
                pm_detail.setTitle(m_loader.getString("IDD_MSG_PROP_DETAIL"));                  
                pm_sender.setTitle(m_loader.getString("IDD_MSG_PROP_SENDER"));                  
            }
            catch (DisplayManagerException e)
            {
                Monitor.logError("MqMessagePropertiesManager: Failed to create panels " + e );
            }
            
            // Put the pages into an array for return
            PanelManager[] PMArray = {pm_detail, pm_sender};
                                                           
            return PMArray;        
     }
     
    /**
     * Returns a list of ActionListener objects to be added as CommitListeners to this
     * group of pages returned by GetPages.
     *
     * For this sample, a simple CommitListener is shown which writes a simple message to the log.
     *
     * @return    an array of ActionListener objects which are to be notified on a Commit.
     *            return null if there are no Commit Listeners.
     */
     
     public ActionListener[] getCommitListeners()
     {
        Monitor.logError("MqMessagePropertiesManager:  Getting Commit Listeners");
        ActionListener[] al = new ActionListener[1];
        al[0] =  new ActionListener()
        {
            public void actionPerformed(ActionEvent evt)
            {
                Monitor.logError("MqMessagePropertiesManager:  Activating Commit Listeners");
            }
        };
                                                   
       return al;  
     }
     
    /**
     * Returns a list of ActionListener objects to be added as CancelListeners to this
     * group of pages returned by GetPages.
     *
     * For this sample, a simple CancelListener is shown which writes a simple message to the log.
     *
     * @return    an array of ActionListener objects which are to be notified on a Commit.
     *            return null if there are no Commit Listeners.
     */
     public ActionListener[] getCancelListeners()
     {
        Monitor.logError("MqMessagePropertiesManager:  Getting Cancel Listeners");
        ActionListener[] al = new ActionListener[1];
        al[0] =  new ActionListener()
        {
            public void actionPerformed(ActionEvent evt)
            {
                Monitor.logError("MqMessagePropertiesManager:  Activating Cancel Listeners");
            }
        };
                                                   
       return al;  
     }

}
